/*
 * Program to perform automated measurements on all the stack from an AFM image, obtaining areas of each slice and HCFA
 * 
 * Author: Laia Pasquina-Lemonche, University of Sheffield, UK
 * 			
 * Date: 12th May 2020
 */

dir1 = getDirectory("Choose Source Directory");
dir2 = getDirectory("Choose Destination Directory");
list = getFileList(dir1);

//Ask if you want to represent the HFCA or not, asnwer only 'YES' or 'NO' 
//HFCAYN= getString("Do you want to calculate the HCFA? (YES or NO)", "YES");
HFCAYN="YES";

//Ask user number for the nm to pixel conversion of the image and the number of slices 
conversion=getNumber("conversion nm/pixel=", 0.769230769);
number_of_repetitions = getNumber("Number of slice to analyse = ", 255);

//This is to avoid seeing all the actions the system does
setBatchMode(true);

//This is to create the array that will hold the final result and the file where it's going to be stored
HCFA = newArray(number_of_repetitions);
headline = "Depth_slice_number, HCFA";
File.append(headline, dir2+"Final"+".csv"); 

//Create vectors to count of pores by different size and create file to save the data
Yellow = newArray(number_of_repetitions);
Green = newArray(number_of_repetitions);
Magenta = newArray(number_of_repetitions);
Blue = newArray(number_of_repetitions);
Total_count = newArray(number_of_repetitions); //To count all the pores in each slice

headline2 = "Depth_slice_number, Yellow_pores, Green_pores, Magenta_pores , Blue_pores, Total_number_pores";
File.append(headline2, dir2+"ColourCount"+".csv"); 


//This for is to repeat the same process for the number of slices chosen by the user
for(j=0;j<=number_of_repetitions;j++){
	
//CHECK=0; //reset check for the HCFA calculation

//Clean up the roi manager
	if (roiManager("count")>0) {
		roiManager("Deselect");
		roiManager("Delete");
	}

run("Close All"); //close all images
run("Clear Results"); //Clear results table

//Open folder with stack of image already filtered and processed
open(dir1+list[j]);

//Convert image pixel to nm (calculate conversion number manually by diving image size (nm) / pixels
run("Properties...", "channels=1 slices=1 frames=1 unit=nm pixel_width="+conversion+" pixel_height="+conversion+" voxel_depth="+conversion+"");

//threshold image (to convert it to the right format for analysis
setAutoThreshold("Otsu dark");

//apply the threshold, select false to put holes in black and ture to put them in white
setOption("BlackBackground", true);
run("Convert to Mask");

//Obtain the table using RoiManager, Measure
run("Analyze Particles...", "size=2-Infinity display include add");
resetThreshold();

//Visualize nothing
roiManager("Show None");

//if there are no rois, do not try to calculate anything
if (roiManager("count")>0) {
number_of_rois = roiManager("count");

//create variables to count pores in each slice of different colours
yellow_rois = 0;
green_rois = 0;
magenta_rois = 0;
blue_rois = 0;

//Classify rois by area and count how many of each colour are
for (i = 0; i < number_of_rois; i++) {
	//select a ROI
	roiManager("select", i);

	Area = getResult("Area", i);
	
	//give it a colour which depends on its area
	if(Area <= 20){
		Roi.setStrokeColor("yellow");
		Roi.setStrokeWidth(5);
		yellow_rois = yellow_rois + 1;
	}
	if(Area > 20 && Area <= 500){
		Roi.setStrokeColor("green");
		Roi.setStrokeWidth(5);
		green_rois = green_rois + 1;
	}
	if(Area > 500 && Area <= 1500){
		Roi.setStrokeColor("magenta");
		Roi.setStrokeWidth(5);
		magenta_rois = magenta_rois + 1;
	}
	if(Area > 1500) {
		Roi.setStrokeColor("blue");
		Roi.setStrokeWidth(5);
		blue_rois = blue_rois + 1;
	}
	//Add it to the overlay
	Overlay.addSelection();
}
//Save processed image
saveAs("jpeg", dir2+list[j]);

//Allocate the count of pores to the colour vectors created at the beggining and the total count, will be different for each slice
Yellow[j]=yellow_rois;
Green[j]=green_rois;
Magenta[j]=magenta_rois;
Blue[j]=blue_rois;
Total_count[j]=number_of_rois;

//Save all the count values from all the pores in ("ColourCount.csv") that we created at the beggining
contentline2= "" + j + "," +Yellow[j]+ "," +Green[j]+","+ Magenta[j]+ ","+ Blue[j]+","+ Total_count[j]; 
File.append(contentline2, dir2+"ColourCount"+".csv");

//Create an array and allocate all values of Area
to_sort = newArray(number_of_rois);
Total=0;

for (i = 0; i < number_of_rois; i++) {
	
	//select a ROI
	roiManager("select", i);
	
	//Allocate values to new arraw
	Area = getResult("Area", i);
	to_sort[i]=Area;
	
	//Calculate total area (sum of all numbers)
	Total = Total + Area;
}
//Sort the values of area from smaller to larger
Sorted = Array.sort(to_sort);

//Define arrawys needed to calculate cumulative fraction of total area per each slice
cum_value = newArray(number_of_rois+1);
cum_fraction = newArray(number_of_rois+1);
cum_value[0]=Sorted[0];
cum_fraction[0]=cum_value[0]/Total;

for (i = 1; i < number_of_rois; i++) {

	//Calculate cumulative value
	cum_value[i]=cum_value[i-1]+Sorted[i];
	//Calculate cumulative fraction of total area
	cum_fraction[i]=cum_value[i]/Total;

	if(HFCAYN=="YES"){
	//Calculate half of the cumulative fraction of the total area (HCFA) nm^2
		if (cum_fraction[i]>0.40 && cum_fraction[i]<0.60) {
      		HCFA_in = i;
      		HCFA[j] = Sorted[HCFA_in];
      		//CHECK=1;
      	}
	}
 	if(HFCAYN=="NO"){
		//If there is not HCFA put Total number of pores instead
		HCFA[j] = Total;
		}
}

/*f(HFCAY_o_N!=YES){
//If there is not HCFA put Total number of pores instead
		if(CHECK!=1){

		HCFA[j] = Total;

		}
	}*/

//Create final results table with the raw Area, the area sorted and the cumulative fraction for each individual slice
table1="Results_table";
Table.create(table1);

for (i = 0; i < number_of_rois; i++) {

	Area = getResult("Area", i);
	Table.set("Raw Area",i, Area);
	Table.set("Area sorted",i, Sorted[i]);
    Table.set("Cumulative fraction",i, cum_fraction[i]);
	}

saveAs("Results",dir2+File.nameWithoutExtension+".csv");
}
//if there are no rois, put HCFA = 0
	else{
	HCFA[j]=0;
	}
//Save all the HCFA values from all the repetitions in one single file ("Final.csv") that we created at the beggining
if (HCFA[j]>0){
contentline= "" + j + "," + HCFA[j]; 
File.append(contentline, dir2+"Final"+".csv");
}

//Close random table that keeps appearing at each iteration
name = File.nameWithoutExtension+".csv";
x=isOpen(name);
if(x==true){
selectWindow(name);
run("Close");

}

}
//Close Results table at the end
name2 = "Results";
y=isOpen(name2);
if(y==true){
selectWindow(name2);
run("Close");
